#ifndef __HW_TEST_TASK_H
#define __HW_TEST_TASK_H

extern unsigned int hw_test_mode;
extern unsigned char rx_text_buf[32];

extern unsigned int hw_test_init(void);
extern unsigned int hw_test_deinit(void);
extern unsigned int hw_uart_test(void);
extern unsigned int hw_eeprom_test(void);
extern unsigned int hw_nor_flash_test(void);
extern unsigned int hw_sdram_test(void);
extern void hw_print_test(void);
extern void hw_relay_test(void);
extern void hw_lcd_test(void);

#endif
